	PROGRAM ABSRISK
C
C-Description-----------------------------------------------------------
C
C  Function:
C    This MS-DOS program uses age-specific mortality and 
C    morbidity data to convert relative risk estimates into 
C    absolute risk estimates.  That is, it estimates the 
C    probability that a patient will suffer a specific morbid or 
C    mortal outcome in a given time interval.  The user first 
C    specifies a data file that contains the needed mortality and 
C    morbidity data for the disease of interest.  She then gives 
C    her patient's age and relative risk, and the time interval 
C    over which the risk estimate is to be derived.  The program 
C    derives this risk, which is given both interactively and in 
C    a log file.
C
C    The method used in this program is described in, Dupont, WD:
C    "Converting relative risks to absolute risks:  a graphical
C    approach".  Statistics in Medicine. 1989; 8:641-651.
C
C  Input:
C
C    The ABSRISK program requires an input data file that 
C    specifies the age-specific morbidity and mortality rates 
C    that are needed for the disease of interest.
C    You must write your own 
C    input data file using an editor or word processor that can 
C    create text files.  An arbitrary number of comment lines can 
C    be included in these files by typing an exclamation point 
C    (!) in column 1.  These lines will be written to the 
C    program's log file and are useful to document the source of 
C    your age-specific data.  The data lines consist of 5 
C    columns separated by blanks that give the following:
C    
C    1.  The start of an age interval.
C    2.  The end of and age interval.
C    3.  The total age-specific mortality rate for the 
C        population of interest in the age group specified in 
C        columns 1 and 2.
C    4.  The corresponding age-specific mortality rate due to 
C        the cause of interest.
C    5.  The corresponding age-specific morbidity rate due to 
C        the cause of interest.
C    
C    The age intervals must be consecutive and of equal length.  
C    If necessary, the program will use linear extrapolation from 
C    the rates of the last two age intervals to estimate rates 
C    for older ages.  The start of the first age interval must be 
C    younger than the youngest patient for whom you wish to make 
C    absolute risk estimates.
C    
C    This program can also be used to estimate the absolute risk 
C    of a mortal event.  In this case the input data file is 
C    created in the same way as before, only now, columns 4 and 5 
C    give identical age-specific mortality rates.
C
C    The program will prompt the user for the name of the input file.
C    It will then prompt the user to enter a patient age and risk interval.
C
C  Output:
C
C    The program writes absolute risk estimates to the users
C    screen and to a log file.  The name of the log file is
C    formed using the root of the input file name and an
C    extension of ".LOG".
C
C  Notes:
C   .   Designed by William D. Dupont
C   .   This routine was written by Dale Plummer
C
C-Declarations----------------------------------------------------------
C
	IMPLICIT NONE
	INCLUDE 'ABSRISK.INC'
C
C  Locals
C
	INTEGER PRVIDED,MAXAGE,MOREYRS,MOREGRPS
	REAL XDELTA,RR,AGE,YEARS
	INTEGER I,MAXINP,STRLEN,INPLUN
	REAL D(200),DELTA(200),PREV
	INTEGER START(200), END(200), SAGE, EAGE
	INTEGER LOGLUN,LEN,LOGLEN
	PARAMETER (LOGLUN=36)
	CHARACTER INFILE*80, LOGNAME*80
	CHARACTER INREC*78, FIXRSK*1
	INTEGER INTERPRE, ISTAT
	REAL ARRAY(5)
C
C-Code------------------------------------------------------------------
C
C    Announce yourself.
C
	WRITE(UNIT=6,FMT='(3(/A))')
     .' This program estimates absolute risks from relative risks and',
     .' age-specific morbidity and mortality data.'//
     .'  See pub/biostat/absrisk.txt',
     .' on the ftp.vanderbilt.edu server for documentation'//
     .' on using this program.'
C
C    Get name of the input data file.
C
	WRITE(UNIT=6,FMT='(/A,$)')
     .' Enter the name of the input data file:  '
	READ(UNIT=5,FMT='(A)',END=9010) INFILE
	CALL UPCASE(INFILE,INFILE)
	IF (INFILE.EQ.' ') INFILE='INFILE'
	LEN=STRLEN(INFILE)
C
C    Try to open the input file.
C
	INPLUN=22
	OPEN(UNIT=INPLUN,FILE=INFILE(1:LEN),STATUS='OLD')
C
C    Form the output file name from the input name with the extension .LOG.
C
	CALL PRSNAM(INFILE,LOGNAME,LOGLEN)
	LOGNAME=INFILE(1:LOGLEN)//'.LOG'
	LOGLEN=STRLEN(LOGNAME)
C
C    Open the log file.
C
	OPEN(UNIT=LOGLUN,FILE=LOGNAME,STATUS='NEW')
C
C    Write title, date, and time to log file.
C
	WRITE(UNIT=LOGLUN,FMT='(/A)')
     .      'ABSRISK program log file.'
c
C    See if the user wants to correct for prevalance of the risk
C    factor in the general population.
C
4       CONTINUE
	WRITE(UNIT=6,FMT='(/A)')
     .' Do you want to correct absolute risks for prevalance of'
	WRITE(UNIT=6,FMT='(A,$)')
     .' risk factor in the general population? (y=yes, n=no, h=help):  '
	READ(UNIT=5,FMT='(A)',END=9010) FIXRSK
	CALL UPCASE(FIXRSK,FIXRSK)
	IF (FIXRSK.NE.'Y'.AND.FIXRSK.NE.'N') FIXRSK='H'
	IF (FIXRSK.EQ.'H') THEN
	    WRITE(UNIT=6,FMT='(/A)')
     .' Answer yes if:'
	    WRITE(UNIT=6,FMT='(A)')
     .'     1.  Relative risk estimates are calculated with respect'
	    WRITE(UNIT=6,FMT='(A)')
     .'         to unexposed subjects'
	    WRITE(UNIT=6,FMT='(3A)')
     .'     2.  Age specific rates in ',INFILE(:LEN),
     .' are from a population '
	    WRITE(UNIT=6,FMT='(A)')
     .'         of both exposed and unexposed subjects'
	    WRITE(UNIT=6,FMT='(A)')
     .'     3.  The exposure prevalance in this population'//
     .' is substantial'
	    GO TO 4
	END IF
C
C    Initialize count of input records.
C
	MAXINP=0
C
C    Look at the input line.  If it begins with "!" then it is a comment.
C    If it is a comment the echo it to the log file.  If it is not a comment
C    then pass it on for processing.
C
	I=1
1       CONTINUE
	    READ(UNIT=INPLUN,FMT='(A)',END=2) INREC
C
C    Echo the input lines to the console and the log file.
C
	    WRITE(UNIT=LOGLUN,FMT='(A)') INREC
	    WRITE(UNIT=6,FMT='(1X,A)') INREC
C
C    If line is not a comment then copy the data to appropriate arrays.
C
	    IF (INREC(1:1) .NE. '!') THEN
		ISTAT=INTERPRE(INREC,ARRAY,5)
		IF (ISTAT.NE.0) GO TO 1091
		START(I)=ARRAY(1)
		END(I)=ARRAY(2)
		D(I)=ARRAY(3)
		DELTA(I)=ARRAY(4)
		C(I)=ARRAY(5)
		DMINUS(I)=D(I)-DELTA(I)
		I=I+1
	    END IF
	GO TO 1
2       CONTINUE
C
C    Done with the input file, close it.
C
	CLOSE(UNIT=INPLUN,STATUS='KEEP')
C
C    Save the count of input data records.
C
	MAXINP=I
C
C    Calculate the starting age FAGE and the width of the age groups W.
C
	W=(END(1)-START(1))+1
	FAGE=START(1)+((W-1)/2)
C
C    Check the age ranges.  The end of interval I must be one less than
C    the start of interval I+1.  The intervals must all be the same width.
C
	DO I=1,MAXINP-1
	    IF ((END(I)-START(I))+1 .NE. W) THEN
		WRITE(UNIT=6,FMT='(/A,I3,A,/,A,I3,A,I3,A)')
     .' Age intervals must be ',W,
     .' years (based on first age interval).',
     .' Length of interval ',I,' is ',(END(I)-START(I))+1,'.'
		GOTO 9010
	    END IF
	END DO

	DO I=1,MAXINP-2
	    IF (START(I+1) .NE. END(I)+1) THEN
		WRITE(UNIT=6,FMT='(/A)')
     .' The start of each interval must be one plus the'//
     .' end of the previous interval.'
		GOTO 9010
	    END IF
	END DO
C
C    We want to have data up to the 105-109 age group (this assumes a
C    group width of 5 years, ie w=5, adjust for other w).  If data is
C    not complete then extrapolate.
C
C    How many age groups were provided?
C
	PRVIDED=MAXINP-1
C
C    Up to what age group was data provided?
C
	MAXAGE=((PRVIDED-1)*W)+(FAGE-INT(W/2))
C
C    How many years do we need to extrapolate?
C
	MOREYRS=105-MAXAGE
C
C    How many more age groups do we need?
C
	MOREGRPS=MOREYRS/W
C
C    If we need to extrapolate then do so here.
C
	IF (MOREGRPS.GT.0) THEN
	    DO I=PRVIDED+1,PRVIDED+MOREGRPS+1
		XDELTA=D(I-1)-D(I-2)
		D(I)=D(I-1)+XDELTA
	    END DO
	    DO I=PRVIDED+1,PRVIDED+MOREGRPS+1
		XDELTA=C(I-1)-C(I-2)
		C(I)=C(I-1)+XDELTA
	    END DO
	    DO I=PRVIDED+1,PRVIDED+MOREGRPS+1
		XDELTA=DELTA(I-1)-DELTA(I-2)
		DELTA(I)=DELTA(I-1)+XDELTA
		DMINUS(I)=D(I)-DELTA(I)
	    END DO
	END IF
C
C    Write the extrapolated cases, if any, to the console and log file.
C
	IF (MOREGRPS.GT.0) THEN
	    WRITE(UNIT=LOGLUN,FMT='(T5,/,A,/)')
     .      ' ------Extrapolated values follow--------------------'
	    WRITE(UNIT=6,FMT='(T5,/,A,/)')
     .      ' ------Extrapolated values follow--------------------'
	    DO 8 I=PRVIDED+1,PRVIDED+MOREGRPS+1
C
C    Since we are restricting the top age to 100 in the input above then
C    we shouldn't print any extrapolated values above 100 here.
C
		SAGE=((I-1)*W)+(FAGE-INT(W/2))
		EAGE=SAGE+W
		IF (EAGE.LE.100) THEN
		    WRITE(UNIT=LOGLUN,FMT='(4X,I3,5X,I3,3(10X,F8.1))')
     .              SAGE,EAGE,D(I),DELTA(I),C(I)
		    WRITE(UNIT=6,FMT='(4X,I3,5X,I3,3(10X,F8.1))')
     .              SAGE,EAGE,D(I),DELTA(I),C(I)
		END IF
8           CONTINUE
	END IF
C
C    Get patient's age and relative risk from user.
C
10      CONTINUE
	WRITE(UNIT=6,FMT='(/A,$)')' Enter patient''s age:  '
	INREC=' '
C
C    Read the user's input.
C
	READ(UNIT=5,FMT='(A)',END=20) INREC
C
C    Interpret the input.
C
	ISTAT=INTERPRE(INREC,ARRAY,1)
	IF (ISTAT.EQ.2) THEN
C
C    If user typed "EXIT" or "EX" then do so.
C
	    GO TO 20
	ELSE IF (ISTAT.EQ.1) THEN
C
C    Any other non-numeric input is an error.
C
	    GO TO 1090
	END IF
C
C    Assign the numeric value to the local variable.
C
	AGE=ARRAY(1)
C
C    Check that number is at least in the range that the below
C    format can successfully print it out to the log file.
C
	IF (AGE.GT.999) GO TO 1090
C
C    Copy input to log file.
C
	WRITE(UNIT=LOGLUN,FMT='(/A)')
     .  ' -------------------------------------------------------------'
	WRITE(UNIT=LOGLUN,FMT='(A,I3)')' Patient''s age:  ',INT(AGE)
C
C    Get relative risk for patient.
C
	WRITE(UNIT=6,FMT='(/A,$)')' Enter patient''s relative risk:  '
	INREC=' '
	READ(UNIT=5,FMT='(A)',END=20) INREC
	ISTAT=INTERPRE(INREC,ARRAY,1)
	IF (ISTAT.EQ.2) THEN
	    GO TO 20
	ELSE IF (ISTAT.EQ.1) THEN
	    GO TO 1090
	END IF
	RR=ARRAY(1)
	IF (RR.GT.999) GO TO 1090
	WRITE(UNIT=LOGLUN,FMT='(/A,F6.2)')
     .' Patient''s relative risk:  ',RR
C
C    Get number of years over which absolute risk is to be estimated.
C
	WRITE(UNIT=6,FMT='(/A,$)')
     .  ' Enter number of years over which absolute risk'//
     .' is to be estimated:  '
	INREC=' '
	READ(UNIT=5,FMT='(A)',END=20) INREC
	ISTAT=INTERPRE(INREC,ARRAY,1)
	IF (ISTAT.EQ.2) THEN
	    GO TO 20
	ELSE IF (ISTAT.EQ.1) THEN
	    GO TO 1090
	END IF
	YEARS=ARRAY(1)
	IF (YEARS.GT.999) GO TO 1090
	WRITE(UNIT=LOGLUN,FMT='(/A,I3)')
     .  ' Number of years over which absolute risk is to estimated:  ',
     .  INT(YEARS)
C
C    Check that age and years are legal.
C
	IF (AGE.LT.START(1) .OR. AGE.GT.100) THEN
	    WRITE(UNIT=6,FMT='(/,A,I3,A)')
     .' The age you enter must be between ',START(1),' and 100.'
	    GO TO 10
	END IF
	IF (AGE+YEARS.GT.100) THEN
	    WRITE(UNIT=6,FMT='(/,A)')
     .' The age you enter plus the number of years you enter'//
     .' must total 100 or less.'
	    GO TO 10
	END IF
C
C    If absolute risk is to be corrected then get percent prevalance
C    of risk factor in general population.
C
5       CONTINUE
	IF (FIXRSK.EQ.'Y') THEN
	    WRITE(UNIT=6,FMT='(/A,$)')
     .  ' Enter percent prevalance of risk factor in'//
     .' general population:  '
	    INREC=' '
	    READ(UNIT=5,FMT='(A)',END=20) INREC
	    ISTAT=INTERPRE(INREC,ARRAY,1)
	    IF (ISTAT.EQ.2) THEN
		GO TO 20
	    ELSE IF (ISTAT.EQ.1) THEN
		GO TO 1090
	    END IF
	    PREV=ARRAY(1)
	    IF (PREV.GT.100.OR.PREV.LT.0) THEN
		WRITE(UNIT=6,FMT='(/,A,I3,A)')
     .' Enter the precent prevalance in the range 0 to 100.'
		GO TO 5
	    END IF
	    WRITE(UNIT=LOGLUN,FMT='(/A)')
     .' Absolute risk is corrected for prevalance of risk factor'//
     .' in general population.'
	    WRITE(UNIT=LOGLUN,FMT='(A,F7.2)')
     .  ' Percent prevalance of risk factor in general population:  ',
     .      PREV
	END IF
C
C    Calculate P(a,s) and write values to output file.
C
	CALL CALPAS(AGE,RR,YEARS,LOGLUN,FIXRSK,PREV)
	GO TO 10
C
C    Done.
C
20      CONTINUE
	WRITE(UNIT=6,FMT='(/A,A,A)')
     .' See '//LOGNAME(1:LOGLEN)//
     .' for a record of the preceding calculations.'
	STOP 'Finished.'
C
C    Come to here on input error.
C
1090    CONTINUE
	WRITE(UNIT=6,FMT='(A)')
     .  ' Input must be a reasonable number or "EXIT".'//
     .'  Please enter again.'
	GO TO 10
C
C    Come to here on error in input file.
C
1091    CONTINUE
	WRITE(UNIT=6,FMT='(A)')
     .  ' Input file data error.  Please check input file format.'
	STOP'[Aborted]'
C
C    Come to here if user types a control-z file name prompt.
C
9010    CONTINUE
	STOP'[Aborted]'
	END
C
C
	SUBROUTINE PRSNAM(FILE,RTNAME,RTLEN)
	IMPLICIT NONE
C
C  Strip extension from file name if present
C
C  Input:   FILE   (Char*(*)) - File name provided by user
C
C  Output:  RTNAME (Char*(*)) - Root name (without extension)
C           RTLEN  (Integer)  - Length of above
C
C  Author:  WDPlummer
C
C-------------------------------------------------------------------------
C
C    Arguments:
	CHARACTER*(*)   FILE,  RTNAME
	INTEGER         RTLEN
C
C    Locals:
	INTEGER         RTNDX
	INTEGER         STRLEN
C
C Code:
C
C    Check for ".ext" construct
C
	RTNDX = INDEX(FILE,'.')
	IF ( RTNDX .GT. 0 ) THEN
C
C    Handle case of degenerate file name ".ext"
C
	    IF (RTNDX.EQ.1) THEN
		RTNAME=FILE(1:1)
		RTLEN=1
		RETURN
	    END IF
C
C    Strip index and return.
C
	    RTNDX = RTNDX - 1
	    RTNAME = FILE(:RTNDX)
	ELSE
C
C    No extension.  Just return.
C
	    RTNAME = FILE
	END IF
C
C    Determine name length.
C
	RTLEN=STRLEN(RTNAME)
C
C    Done
C
	RETURN
	END
