 	REAL FUNCTION RRISK(RHO,CURAGE,ENTAGE,WHICHF)
C
C-Description-----------------------------------------------------------
C
C  Function:
C	Calculate the relative risk.
C
C  Usage:
C	Absolute risk program
C
C  Input:
C    RHO    is a constant used to define RRISK.  When WHICHF=1, RHO
C           equals the relative risk.
C
C    CURAGE = t in Dupont (1989) is the patient's current age.  RRISK
C           returns the patient's relative risk at age CURAGE.
C
C    ENTAGE = a in DUpont (1989) is the patient's entry age.  That is,
C           her age at the beginning of the risk interval of interest.
C
C    WHICHF is an indicator variable which determines the shape of
C           RRISK.  It can take the values 1, 2, or 3 to produce the
C           following relative risk functions, respectively:
C
C           1  RRISK=RHO
C           2  RRISK has a sigmoidal shape that decreases monotonically
C              with increasing age.  When RHO<=1, RRISK=RHO.  The
C              precise form of this curve is given below.
C           3  RRISK=RHO*((CURAGE/ENTAGE)**3.5).
C
C    The user may wish to modify these definitions according to her needs.
C
C  Output:
C    RRISK  = r(a,t) in Dupont (1989) is the patient's relative risk
C           function for developing the disease of interest.  The
C           shape of this curve is determined by the argument WHICHF.
C
C  Notes:
C   .	This routine will calculate relative risks using either a
C	constant hazard function or a sigmoidal risk function.
C   .	This routine was designed by William D. Dupont.
C   .	This routine was written by Dale Plummer.
C
C-Declarations----------------------------------------------------------
C
	IMPLICIT NONE
C
C  Arguments
C
	REAL RHO,CURAGE,ENTAGE
	INTEGER WHICHF
C
C  Locals
C
	REAL TEMP1,TEMP2,ELAPSD
C
C-Code------------------------------------------------------------------
C
C    Initiaize return value.
C
	RRISK=0.0
C
C    Use the appropriate function.
C
	IF (WHICHF.EQ.1) THEN
C
C    Constant hazard.
C
	    RRISK=RHO
	ELSE IF (WHICHF.EQ.2) THEN
C
C    Sigmoidal shaped hazard function.  Don't do this when rho=1.
C
	    IF (RHO.GT.1) THEN
		ELAPSD=CURAGE-ENTAGE
		TEMP1=(5./3.)*LOG(99.-(100./RHO))
		TEMP2=(-2./15.)*LOG(99.-(100./RHO))
		RRISK=
     .			(((RHO-1.)*EXP(TEMP1+(TEMP2*ELAPSD)))/
     .			(1+EXP(TEMP1+(TEMP2*ELAPSD))))+1.
	    ELSE
C
C    Return constant hazard for sigmoidal case when rho=1.
C
		RRISK=RHO
	    END IF
	ELSE IF (WHICHF.EQ.3) THEN
		RRISK=RHO*((CURAGE/ENTAGE)**3.5)
	ELSE
	    STOP 'Bad function selection in RRISK'
	END IF
C
C    Finished.
C
	RETURN
	END
